local bumperhud = "gametypeinfo"

local restoreHud = false

-- Net vars
local enabled = false
local exiting = false

local cvar_enable = CV_RegisterVar({
    name = "nobumpers",
    defaultvalue = "On",
    flags = CV_NETVAR,
    PossibleValue = CV_OnOff,
})

local function playerDamage(victim, damager)
    if exiting or not enabled then return end

    victim.nbb_score = max(($ or 0) - 1, 0)
    victim.nbb_flash = TICRATE
    victim.nbb_flashgood = false

    if damager and damager ~= victim then
        local inc = K_IsPlayerWanted(victim) and 3 or 1

        damager.nbb_score = ($ or 0) + inc
        damager.nbb_flash = TICRATE
        damager.nbb_flashgood = true
    end
end

local function onDamage(victim, inflictor, source)
    playerDamage(victim, source and source.player)
end

addHook("PlayerSpin", onDamage)
addHook("PlayerExplode", onDamage)
addHook("PlayerSquish", onDamage)
addHook("MobjDeath", function(pmo)
    -- Paranoia
    if not (pmo and pmo.valid and pmo.player) then return end

    playerDamage(pmo.player)
end, MT_PLAYER)

local function isEnabled()
    if (not G_BattleGametype()) then return false end
    if (not cvar_enable.value) then return false end

    local hm_timelimit = CV_FindVar("hm_timelimit")
    if (not hm_timelimit) then
        return false
    end
    return hm_timelimit.value > 0
end

local function setEnabled()
    enabled = isEnabled()
    exiting = false

    for p in players.iterate do
        p.nbb_score = 0
    end
end

rawset(_G, "G_NoBumperBattleGametype", function()
    return enabled
end)

local function hideBumpers(obj)
    if (not enabled) then return end

    obj.sprite = SPR_NULL
    return true
end

local function setWanted()
    local maxScore = 0
    local playerScore
    for p in players.iterate do
        if (p == nil or p.mo == nil) then continue end
        playerScore = p.marescore
        maxScore = (maxScore > playerScore) and maxScore or playerScore
    end

    local wantedScore = 1000
    for p in players.iterate do
        if (p == nil or p.mo == nil) then continue end
        if (p.marescore == maxScore) then
            p.kartstuff[k_wanted] = wantedScore
            wantedScore = $ - 1
        else
            p.kartstuff[k_wanted] = 0
        end
    end
end

local function bumperAndScoreLogic()
    if (not enabled) then
        if restoreHud then
            hud.enable(bumperhud)
            restoreHud = false
        end
        return
    end

    hud.disable(bumperhud)
    restoreHud = true

    exiting = true

    setWanted()
    for p in players.iterate do
        if p.spectator then continue end

        if not p.exiting then exiting = false end

        if (p.kartstuff[k_bumper] ~= 3) then
            p.kartstuff[k_bumper] = 3
        end

        p.marescore = p.nbb_score or 0

        if p.nbb_flash then p.nbb_flash = $ - 1 end
    end
end

local function netVars(n)
    enabled = n($)
    exiting = n($)
end

local lapn_patch, lapn_patch_clr
local function drawPoints(v, p)
    if p.spectator then return end
    if not lapn_patch then lapn_patch = v.cachePatch("K_STLAPN") end

    if (not enabled) then return end

    local splitnum = nil
    for i = 0, splitscreen do
        if displayplayers[i] == p then
            splitnum = i
        end
    end

    local x, y

    local flags = V_HUDTRANS | V_SNAPTOLEFT | V_SNAPTOBOTTOM

    if splitscreen > 0 then
        if splitscreen == 1 then
            x = 20
            y = ((splitnum == 0) and 100 or 200) - 40
        else
            x = ((splitnum == 0) or splitnum == 2) and 20 or 180
            y = (((splitnum == 0) or splitnum == 1) and 100 or 200) - 40
        end
    else
        x = 20
        y = 200 - 38
    end

    local colormap
    local patch = lapn_patch

    if v.useColorHud() then
        if not lapn_patch_clr then lapn_patch_clr = v.cachePatch("K_SCLAPN") end

        patch = lapn_patch_clr

        local color = v.getHudColor()

        colormap = v.getColormap(TC_DEFAULT, color)
    end

    if splitscreen < 2 then
        v.draw(x, y, patch, flags, colormap)
    end

    local textflags = flags

    if p.nbb_flash and (p.nbb_flash / (TICRATE/8)) % 2 == 0 then
        textflags = textflags | (p.nbb_flashgood and V_GREENMAP or V_REDMAP)
    end

    v.drawString(x+14, y+6, string.format("%02d pts", p.marescore), textflags, splitscreen > 1 and "small" or "thin")
end

addHook("NetVars", netVars)
addHook("MapLoad", setEnabled)
addHook("ThinkFrame", bumperAndScoreLogic)
addHook("MobjThinker", hideBumpers, MT_BATTLEBUMPER)
hud.add(drawPoints)
